@@ -163,7 +163,7 @@ def get_plugin_info(env, include_core=False):
                         if k == 'home_page' or k == 'url':
                             k = 'home_page'
                             v = v.replace('$', '').replace('URL: ', '') 
-                        if k == 'author':
+                        else:
                             v = to_unicode(v)
                         info[k] = v
             else:
@@ -173,7 +173,7 @@ def get_plugin_info(env, include_core=False):
                 for k in info:
                     if info[k] == 'UNKNOWN':
                         info[k] = ''
-                    elif k == 'author':
+                    else:
                         # Must be encoded as unicode as otherwise Genshi 
                         # may raise a "UnicodeDecodeError".
                         info[k] = to_unicode(info[k])
