@@ -25,8 +25,10 @@ from trac.perm import PermissionSystem
 from trac.env import IEnvironmentSetupParticipant
 from trac.config import Configuration
 from trac.ticket.api import ITicketActionController, TicketSystem
-from trac.util.compat import set
-from trac.util.translation import _
+from trac.ticket.model import Resolution
+from trac.util.text import obfuscate_email_address
+from trac.util.translation import _, tag_
+from trac.web.chrome import Chrome
 
 # -- Utilities for the ConfigurableTicketWorkflow
 
@@ -98,8 +100,10 @@ def load_workflow_config_snippet(config, filename):
 
 class ConfigurableTicketWorkflow(Component):
     """Ticket action controller which provides actions according to a
-    workflow defined in the TracIni configuration file, inside the
-    [ticket-workflow] section.
+    workflow defined in trac.ini.
+    
+    The workflow is idefined in the `[ticket-workflow]` section of the
+    [wiki:TracIni#ticket-workflow-section trac.ini] configuration file.
     """
     
     def __init__(self, *args, **kwargs):
@@ -171,30 +175,31 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
         # once and get really confused.
         status = ticket._old.get('status', ticket['status']) or 'new'
 
+        ticket_perm = req.perm(ticket.resource)
         allowed_actions = []
         for action_name, action_info in self.actions.items():
             oldstates = action_info['oldstates']
             if oldstates == ['*'] or status in oldstates:
                 # This action is valid in this state.  Check permissions.
-                allowed = 0
                 required_perms = action_info['permissions']
-                if required_perms:
-                    for permission in required_perms:
-                        if permission in req.perm(ticket.resource):
-                            allowed = 1
-                            break
-                else:
-                    allowed = 1
-                if allowed:
+                if self._is_action_allowed(ticket_perm, required_perms):
                     allowed_actions.append((action_info['default'],
                                             action_name))
         if not (status in ['new', 'closed'] or \
                     status in TicketSystem(self.env).get_all_status()) \
-                and 'TICKET_ADMIN' in req.perm(ticket.resource):
+                and 'TICKET_ADMIN' in ticket_perm:
             # State no longer exists - add a 'reset' action if admin.
             allowed_actions.append((0, '_reset'))
         return allowed_actions
 
+    def _is_action_allowed(self, ticket_perm, required_perms):
+        if not required_perms:
+            return True
+        for permission in required_perms:
+            if permission in ticket_perm:
+                return True
+        return False
+
     def get_all_status(self):
         """Return a list of all states described by the configuration.
 
@@ -207,7 +212,6 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
         return all_status
         
     def render_ticket_action_control(self, req, ticket, action):
-        from trac.ticket import model
 
         self.log.debug('render_ticket_action_control: action "%s"' % action)
 
@@ -215,6 +219,12 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
         status = this_action['newstate']        
         operations = this_action['operations']
         current_owner = ticket._old.get('owner', ticket['owner'] or '(none)')
+        if not (Chrome(self.env).show_email_addresses
+                or 'EMAIL_VIEW' in req.perm(ticket.resource)):
+            format_user = obfuscate_email_address
+        else:
+            format_user = lambda address: address
+        current_owner = format_user(current_owner)
 
         control = [] # default to nothing
         hints = []
@@ -239,27 +249,35 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
 
             if owners == None:
                 owner = req.args.get(id, req.authname)
-                control.append(tag(['to ', tag.input(type='text', id=id,
-                                                     name=id, value=owner)]))
-                hints.append(_("The owner will change from %(current_owner)s",
+                control.append(tag_('to %(owner)s',
+                                    owner=tag.input(type='text', id=id,
+                                                    name=id, value=owner)))
+                hints.append(_("The owner will be changed from "
+                               "%(current_owner)s",
                                current_owner=current_owner))
             elif len(owners) == 1:
-                control.append(tag('to %s ' % owners[0]))
+                owner = tag.input(type='hidden', id=id, name=id,
+                                  value=owners[0])
+                formatted_owner = format_user(owners[0])
+                control.append(tag_('to %(owner)s ',
+                                    owner=tag(formatted_owner, owner)))
                 if ticket['owner'] != owners[0]:
-                    hints.append(_("The owner will change from "
+                    hints.append(_("The owner will be changed from "
                                    "%(current_owner)s to %(selected_owner)s",
                                    current_owner=current_owner,
-                                   selected_owner=owners[0]))
+                                   selected_owner=formatted_owner))
             else:
-                control.append(tag([_("to "), tag.select(
-                    [tag.option(x, selected=(x == selected_owner or None))
+                control.append(tag_('to %(owner)s', owner=tag.select(
+                    [tag.option(format_user(x), value=x,
+                                selected=(x == selected_owner or None))
                      for x in owners],
-                    id=id, name=id)]))
-                hints.append(_("The owner will change from %(current_owner)s",
+                    id=id, name=id)))
+                hints.append(_("The owner will be changed from "
+                               "%(current_owner)s",
                                current_owner=current_owner))
         if 'set_owner_to_self' in operations and \
                 ticket._old.get('owner', ticket['owner']) != req.authname:
-            hints.append(_("The owner will change from %(current_owner)s "
+            hints.append(_("The owner will be changed from %(current_owner)s "
                            "to %(authname)s", current_owner=current_owner,
                            authname=req.authname))
         if 'set_resolution' in operations:
@@ -267,31 +285,39 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
                 resolutions = [x.strip() for x in
                                this_action['set_resolution'].split(',')]
             else:
-                resolutions = [val.name for val in
-                               model.Resolution.select(self.env)]
+                resolutions = [val.name for val in Resolution.select(self.env)]
             if not resolutions:
                 raise TracError(_("Your workflow attempts to set a resolution "
                                   "but none is defined (configuration issue, "
                                   "please contact your Trac admin)."))
+            id = 'action_%s_resolve_resolution' % action
             if len(resolutions) == 1:
-                control.append(tag('as %s' % resolutions[0]))
-                hints.append(_("The resolution will be set to %s") %
-                             resolutions[0])
+                resolution = tag.input(type='hidden', id=id, name=id,
+                                       value=resolutions[0])
+                control.append(tag_('as %(resolution)s',
+                                    resolution=tag(resolutions[0],
+                                                   resolution)))
+                hints.append(_("The resolution will be set to %(name)s",
+                               name=resolutions[0]))
             else:
-                id = 'action_%s_resolve_resolution' % action
-                selected_option = req.args.get(id,
-                        self.config.get('ticket', 'default_resolution'))
-                control.append(tag(['as ', tag.select(
-                    [tag.option(x, selected=(x == selected_option or None))
+                selected_option = req.args.get(id, 
+                        TicketSystem(self.env).default_resolution)
+                control.append(tag_('as %(resolution)s',
+                                    resolution=tag.select(
+                    [tag.option(x, value=x,
+                                selected=(x == selected_option or None))
                      for x in resolutions],
-                    id=id, name=id)]))
+                    id=id, name=id)))
                 hints.append(_("The resolution will be set"))
+        if 'del_resolution' in operations:
+            hints.append(_("The resolution will be deleted"))
         if 'leave_status' in operations:
-            control.append('as %s ' % ticket._old.get('status', 
-                                                      ticket['status']))
+            control.append(_('as %(status)s ',
+                             status= ticket._old.get('status',
+                                                     ticket['status'])))
         else:
             if status != '*':
-                hints.append(_("Next status will be '%s'") % status)
+                hints.append(_("Next status will be '%(name)s'", name=status))
         return (this_action['name'], tag(*control), '. '.join(hints))
 
     def get_ticket_changes(self, req, ticket, action):
@@ -312,7 +338,7 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
         for operation in this_action['operations']:
             if operation == 'reset_workflow':
                 updated['status'] = 'new'
-            if operation == 'del_owner':
+            elif operation == 'del_owner':
                 updated['owner'] = ''
             elif operation == 'set_owner':
                 newowner = req.args.get('action_%s_reassign_owner' % action,
@@ -324,8 +350,7 @@ Read TracWorkflow for more information (don't forget to 'wiki upgrade' as well)
                 updated['owner'] = newowner
             elif operation == 'set_owner_to_self':
                 updated['owner'] = req.authname
-
-            if operation == 'del_resolution':
+            elif operation == 'del_resolution':
                 updated['resolution'] = ''
             elif operation == 'set_resolution':
                 newresolution = req.args.get('action_%s_resolve_resolution' % \
