@@ -18,7 +18,7 @@
 #         Christopher Lenz <cmlenz@gmx.de>
 
 import re
-from datetime import date, datetime
+from datetime import datetime
 
 from trac.attachment import Attachment
 from trac.core import TracError
@@ -33,6 +33,15 @@ __all__ = ['Ticket', 'Type', 'Status', 'Resolution', 'Priority', 'Severity',
            'Component', 'Milestone', 'Version', 'group_milestones']
 
 
+def _fixup_cc_list(cc_value):
+    """Fix up cc list separators and remove duplicates."""
+    cclist = []
+    for cc in re.split(r'[;,\s]+', cc_value):
+        if cc and cc not in cclist:
+            cclist.append(cc)
+    return ', '.join(cclist)
+
+
 class Ticket(object):
 
     # Fields that must not be modified directly by the user
@@ -45,7 +54,7 @@ class Ticket(object):
     # 0.11 compatibility
     time_created = property(lambda self: self.values.get('time'))
     time_changed = property(lambda self: self.values.get('changetime'))
-    
+
     def __init__(self, env, tkt_id=None, db=None, version=None):
         self.env = env
         if tkt_id is not None:
@@ -158,7 +167,7 @@ class Ticket(object):
                 return field[0].get('value', '')
         except KeyError:
             pass
-        
+
     def populate(self, values):
         """Populate the ticket with 'suitable' values from a dictionary"""
         field_names = [f['name'] for f in self.fields]
@@ -173,11 +182,14 @@ class Ticket(object):
 
     def insert(self, when=None, db=None):
         """Add ticket to database.
-        
+
         The `db` argument is deprecated in favor of `with_transaction()`.
         """
         assert not self.exists, 'Cannot insert an existing ticket'
 
+        if 'cc' in self.values:
+            self['cc'] = _fixup_cc_list(self.values['cc'])
+
         # Add a timestamp
         if when is None:
             when = datetime.now(utc)
@@ -198,7 +210,7 @@ class Ticket(object):
         for field in self.time_fields:
             if field in values:
                 values[field] = to_utimestamp(values[field])
-        
+
         # Insert ticket record
         std_fields = []
         custom_fields = []
@@ -240,11 +252,14 @@ class Ticket(object):
         Store ticket changes in the database. The ticket must already exist in
         the database.  Returns False if there were no changes to save, True
         otherwise.
-        
+
         The `db` argument is deprecated in favor of `with_transaction()`.
         """
         assert self.exists, 'Cannot update a new ticket'
 
+        if 'cc' in self.values:
+            self['cc'] = _fixup_cc_list(self.values['cc'])
+
         if not self._old and not comment:
             return False # Not modified
 
@@ -271,14 +286,6 @@ class Ticket(object):
                     # we just leave the owner as is.
                     pass
 
-        # Fix up cc list separators and remove duplicates
-        if 'cc' in self.values:
-            cclist = []
-            for cc in re.split(r'[;,\s]+', self.values['cc']):
-                if cc not in cclist:
-                    cclist.append(cc)
-            self.values['cc'] = ', '.join(cclist)
-
         @self.env.with_transaction(db)
         def do_save(db):
             cursor = db.cursor()
@@ -333,7 +340,7 @@ class Ticket(object):
                     VALUES (%s, %s, %s, %s, %s, %s)
                     """, (self.id, when_ts, author, name, self._old[name],
                           self[name]))
-            
+
             # always save comment, even if empty 
             # (numbering support for timeline)
             cursor.execute("""
@@ -341,7 +348,7 @@ class Ticket(object):
                     (ticket,time,author,field,oldvalue,newvalue)
                 VALUES (%s,%s,%s,'comment',%s,%s)
                 """, (self.id, when_ts, author, comment_num, comment))
-    
+
             cursor.execute("UPDATE ticket SET changetime=%s WHERE id=%s",
                            (when_ts, self.id))
 
@@ -371,22 +378,22 @@ class Ticket(object):
                 FROM ticket_change WHERE ticket=%s AND time=%s 
                   UNION 
                 SELECT time,author,'attachment',null,filename, 0 AS permanent
-                FROM attachment WHERE id=%s AND time=%s 
+                FROM attachment WHERE type='ticket' AND id=%s AND time=%s 
                   UNION 
                 SELECT time,author,'comment',null,description, 0 AS permanent
-                FROM attachment WHERE id=%s AND time=%s
+                FROM attachment WHERE type='ticket' AND id=%s AND time=%s
                 ORDER BY time,permanent,author
                 """, (self.id, when_ts, sid, when_ts, sid, when_ts))
         else:
             cursor.execute("""
                 SELECT time,author,field,oldvalue,newvalue, 1 AS permanent
-                FROM ticket_change WHERE ticket=%s
+                FROM ticket_change WHERE ticket=%s 
                   UNION 
                 SELECT time,author,'attachment',null,filename, 0 AS permanent
-                FROM attachment WHERE id=%s 
+                FROM attachment WHERE type='ticket' AND id=%s 
                   UNION 
                 SELECT time,author,'comment',null,description, 0 AS permanent
-                FROM attachment WHERE id=%s 
+                FROM attachment WHERE type='ticket' AND id=%s 
                 ORDER BY time,permanent,author
                 """, (self.id, sid, sid))
         log = []
@@ -397,7 +404,7 @@ class Ticket(object):
 
     def delete(self, db=None):
         """Delete the ticket.
-        
+
         The `db` argument is deprecated in favor of `with_transaction()`.
         """
         @self.env.with_transaction(db)
@@ -440,7 +447,7 @@ class Ticket(object):
             if not row:
                 return
             ts = row[0]
-            
+
             custom_fields = set(f['name'] for f in self.fields
                                 if f.get('custom'))
 
@@ -478,12 +485,12 @@ class Ticket(object):
                         cursor.execute("""
                             UPDATE ticket SET %s=%%s WHERE id=%%s
                             """ % field, (oldvalue, self.id))
-            
+
             # Delete the change
             cursor.execute("""
                 DELETE FROM ticket_change WHERE ticket=%s AND time=%s
                 """, (self.id, ts))
-            
+
             # Fix the last modification time
             cursor.execute("""
                 UPDATE ticket SET changetime=(
@@ -491,7 +498,7 @@ class Ticket(object):
                     ORDER BY time DESC LIMIT 1)
                 WHERE id=%s
                 """, (self.id, self.id))
-        
+
         self._fetch_ticket(self.id)
 
     def modify_comment(self, cdate, author, comment, when=None):
@@ -516,7 +523,7 @@ class Ticket(object):
                 break
             if comment == (old_comment or ''):
                 return
-        
+
             # Comment history is stored in fields named "_comment%d"
             # Find the next edit number
             cursor.execute("""
@@ -593,7 +600,7 @@ class Ticket(object):
                               '%' + db.like_escape('.' + scnum)))
         for row in cursor:
             return row
-        
+
         # Fallback when comment number is not available in oldvalue
         num = 0
         cursor.execute("""
@@ -618,7 +625,7 @@ class Ticket(object):
                 break
         else:
             return
-        
+
         # Find author if NULL
         if author is None:
             cursor.execute("""
@@ -630,7 +637,6 @@ class Ticket(object):
                 break
         return (ts, author, comment)
 
-
 def simplify_whitespace(name):
     """Strip spaces and remove duplicate spaces within names"""
     if name:
@@ -666,7 +672,7 @@ class AbstractEnum(object):
 
     def delete(self, db=None):
         """Delete the enum value.
-        
+
         The `db` argument is deprecated in favor of `with_transaction()`.
         """
         assert self.exists, 'Cannot delete non-existent %s' % self.type
@@ -692,7 +698,7 @@ class AbstractEnum(object):
 
     def insert(self, db=None):
         """Add a new enum value.
-        
+
         The `db` argument is deprecated in favor of `with_transaction()`.
         """
         assert not self.exists, 'Cannot insert existing %s' % self.type
@@ -719,7 +725,7 @@ class AbstractEnum(object):
 
     def update(self, db=None):
         """Update the enum value.
-        
+
         The `db` argument is deprecated in favor of `with_transaction()`.
         """
         assert self.exists, 'Cannot update non-existent %s' % self.type
@@ -791,7 +797,6 @@ class Severity(AbstractEnum):
 
 
 class Component(object):
-
     def __init__(self, env, name=None, db=None):
         self.env = env
         if name:
@@ -817,7 +822,7 @@ class Component(object):
 
     def delete(self, db=None):
         """Delete the component.
-        
+
         The `db` argument is deprecated in favor of `with_transaction()`.
         """
         assert self.exists, 'Cannot delete non-existent component'
@@ -832,7 +837,7 @@ class Component(object):
 
     def insert(self, db=None):
         """Insert a new component.
-        
+
         The `db` argument is deprecated in favor of `with_transaction()`.
         """
         assert not self.exists, 'Cannot insert existing component'
@@ -853,7 +858,7 @@ class Component(object):
 
     def update(self, db=None):
         """Update the component.
-        
+
         The `db` argument is deprecated in favor of `with_transaction()`.
         """
         assert self.exists, 'Cannot update non-existent component'
@@ -894,7 +899,6 @@ class Component(object):
 
 
 class Milestone(object):
-
     def __init__(self, env, name=None, db=None):
         self.env = env
         if name:
@@ -925,8 +929,8 @@ class Milestone(object):
 
     exists = property(fget=lambda self: self._old['name'] is not None)
     is_completed = property(fget=lambda self: self.completed is not None)
-    is_late = property(fget=lambda self: self.due and \
-                                         self.due.date() < date.today())
+    is_late = property(fget=lambda self: self.due and
+                                         self.due < datetime.now(utc))
 
     def _from_database(self, row):
         name, due, completed, description = row
@@ -943,7 +947,7 @@ class Milestone(object):
 
     def delete(self, retarget_to=None, author=None, db=None):
         """Delete the milestone.
-        
+
         The `db` argument is deprecated in favor of `with_transaction()`.
         """
         @self.env.with_transaction(db)
@@ -970,7 +974,7 @@ class Milestone(object):
 
     def insert(self, db=None):
         """Insert a new milestone.
-        
+
         The `db` argument is deprecated in favor of `with_transaction()`.
         """
         self.name = simplify_whitespace(self.name)
@@ -994,7 +998,7 @@ class Milestone(object):
 
     def update(self, db=None):
         """Update the milestone.
-        
+
         The `db` argument is deprecated in favor of `with_transaction()`.
         """
         self.name = simplify_whitespace(self.name)
@@ -1071,7 +1075,6 @@ def group_milestones(milestones, include_completed):
 
 
 class Version(object):
-
     def __init__(self, env, name=None, db=None):
         self.env = env
         if name:
@@ -1097,7 +1100,7 @@ class Version(object):
 
     def delete(self, db=None):
         """Delete the version.
-        
+
         The `db` argument is deprecated in favor of `with_transaction()`.
         """
         assert self.exists, 'Cannot delete non-existent version'
@@ -1112,7 +1115,7 @@ class Version(object):
 
     def insert(self, db=None):
         """Insert a new version.
-        
+
         The `db` argument is deprecated in favor of `with_transaction()`.
         """
         assert not self.exists, 'Cannot insert existing version'
@@ -1132,7 +1135,7 @@ class Version(object):
 
     def update(self, db=None):
         """Update the version.
-        
+
         The `db` argument is deprecated in favor of `with_transaction()`.
         """
         assert self.exists, 'Cannot update non-existent version'
