@@ -38,8 +38,9 @@ from trac.util.text import empty, shorten_line, unicode_unquote
 from trac.util.translation import _, tag_
 from trac.web import arg_list_to_args, parse_arg_list, IRequestHandler
 from trac.web.href import Href
-from trac.web.chrome import add_ctxtnav, add_link, add_script, add_stylesheet, \
-                            add_warning, INavigationContributor, Chrome
+from trac.web.chrome import add_ctxtnav, add_link, add_script, \
+                            add_script_data, add_stylesheet, add_warning, \
+                            INavigationContributor, Chrome
 
 from trac.wiki.api import IWikiSyntaxProvider
 from trac.wiki.macros import WikiMacroBase # TODO: should be moved in .api
@@ -322,15 +323,11 @@ class Query(object):
             result = {}
             for i in column_indices:
                 name, field, val = columns[i], fields[i], row[i]
-                if name == self.group:
-                    val = val or 'None'
-                elif name == 'reporter':
+                if name == 'reporter':
                     val = val or 'anonymous'
                 elif name == 'id':
                     val = int(val)
                     result['href'] = req.href.ticket(val)
-                elif val is None:
-                    val = '--'
                 elif name in self.time_fields:
                     val = from_utimestamp(val)
                 elif field and field['type'] == 'checkbox':
@@ -338,6 +335,8 @@ class Query(object):
                         val = bool(int(val))
                     except (TypeError, ValueError):
                         val = False
+                elif val is None:
+                    val = ''
                 result[name] = val
             results.append(result)
         cursor.close()
@@ -1085,6 +1084,13 @@ class QueryModule(Component):
         data['all_columns'].remove('id')
         data['all_textareas'] = query.get_all_textareas()
 
+        properties = dict((name, dict((key, field[key])
+                                      for key in ('type', 'label', 'options')
+                                      if key in field))
+                          for name, field in data['fields'].iteritems())
+        add_script_data(req, {'properties': properties,
+                              'modes': data['modes']})
+
         add_stylesheet(req, 'common/css/report.css')
         add_script(req, 'common/js/query.js')
 
