@@ -16,7 +16,7 @@
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
 from StringIO import StringIO
-from datetime import datetime
+from datetime import datetime, timedelta
 import re
 
 from genshi.builder import tag
@@ -30,8 +30,8 @@ from trac.perm import IPermissionRequestor
 from trac.resource import *
 from trac.search import ISearchSource, search_to_sql, shorten_result
 from trac.util.datefmt import parse_date, utc, to_utimestamp, \
-                              get_date_format_hint, get_datetime_format_hint, \
-                              format_date, format_datetime, from_utimestamp
+                              get_datetime_format_hint, format_date, \
+                              format_datetime, from_utimestamp
 from trac.util.text import CRLF
 from trac.util.translation import _, tag_
 from trac.ticket import Milestone, Ticket, TicketSystem, group_milestones
@@ -444,8 +444,7 @@ class RoadmapModule(Component):
                 write_date('DTSTART', milestone.due)
                 write_prop('SUMMARY', _('Milestone %(name)s',
                                         name=milestone.name))
-                write_prop('URL', req.base_url + '/milestone/' +
-                           milestone.name)
+                write_prop('URL', req.abs_href.milestone(milestone.name))
                 if milestone.description:
                     write_prop('DESCRIPTION', milestone.description)
                 write_prop('END', 'VEVENT')
@@ -625,8 +624,11 @@ class MilestoneModule(Component):
         
         milestone.description = req.args.get('description', '')
 
-        due = req.args.get('duedate', '')
-        milestone.due = due and parse_date(due, tzinfo=req.tz) or None
+        if 'due' in req.args:
+            due = req.args.get('duedate', '')
+            milestone.due = due and parse_date(due, req.tz, 'datetime') or None
+        else:
+            milestone.due = None
 
         completed = req.args.get('completeddate', '')
         retarget_to = req.args.get('target')
@@ -658,7 +660,8 @@ class MilestoneModule(Component):
 
         # -- check completed date
         if 'completed' in req.args:
-            completed = completed and parse_date(completed, req.tz) or None
+            completed = completed and parse_date(completed, req.tz,
+                                                 'datetime') or None
             if completed and completed > datetime.now(utc):
                 warn(_('Completion date may not be in the future'))
         else:
@@ -701,10 +704,16 @@ class MilestoneModule(Component):
         return 'milestone_delete.html', data, None
 
     def _render_editor(self, req, db, milestone):
+        # Suggest a default due time of 18:00 in the user's timezone
+        default_due = datetime.now(req.tz).replace(hour=18, minute=0, second=0,
+                                                   microsecond=0)
+        if default_due <= datetime.now(utc):
+            default_due += timedelta(days=1)
+        
         data = {
             'milestone': milestone,
-            'date_hint': get_date_format_hint(),
             'datetime_hint': get_datetime_format_hint(),
+            'default_due': default_due,
             'milestone_groups': [],
         }
 
@@ -763,19 +772,24 @@ class MilestoneModule(Component):
             groups = []
             for field in ticket_fields:
                 if field['name'] == by:
-                    if field.has_key('options'):
+                    if 'options' in field:
                         groups = field['options']
+                        if field.get('optional'):
+                            groups.insert(0, '')
                     else:
                         cursor = db.cursor()
-                        cursor.execute("SELECT DISTINCT %s FROM ticket "
-                                       "ORDER BY %s" % (by, by))
+                        cursor.execute("""
+                            SELECT DISTINCT COALESCE(%s,'') FROM ticket
+                            ORDER BY COALESCE(%s,'')
+                            """ % (by, by))
                         groups = [row[0] for row in cursor]
 
             max_count = 0
             group_stats = []
 
             for group in groups:
-                group_tickets = [t for t in tickets if t[by] == group]
+                values = group and (group,) or (None, group)
+                group_tickets = [t for t in tickets if t[by] in values]
                 if not group_tickets:
                     continue
 
