@@ -38,7 +38,7 @@ from trac.util import get_reporter_id
 from trac.util.compat import any
 from trac.util.datefmt import format_datetime, from_utimestamp, \
                               to_utimestamp, utc
-from trac.util.text import exception_to_unicode, obfuscate_email_address,  \
+from trac.util.text import exception_to_unicode, obfuscate_email_address, \
                            shorten_line, to_unicode
 from trac.util.presentation import separated
 from trac.util.translation import _, tag_, tagn_, N_, gettext, ngettext
@@ -228,7 +228,7 @@ class TicketModule(Component):
 
     def get_timeline_filters(self, req):
         if 'TICKET_VIEW' in req.perm:
-            yield ('ticket', _("Opened and closed tickets"))
+            yield ('ticket', _("Tickets opened and closed"))
             if self.timeline_details:
                 yield ('ticket_details', _("Ticket updates"), False)
 
@@ -291,6 +291,9 @@ class TicketModule(Component):
                 """ % (ts_start, ts_stop))
             data = None
             for id,t,author,type,summary,field,oldvalue,newvalue in cursor:
+                if not (oldvalue or newvalue):
+                    # ignore empty change from custom field created or deleted
+                    continue 
                 if not data or (id, t) != data[:2]:
                     if data:
                         ev = produce_event(data, status, fields, comment, cid)
@@ -920,7 +923,7 @@ class TicketModule(Component):
 
     def _render_comment_diff(self, req, ticket, data, cnum):
         """Show differences between two versions of a ticket comment."""
-        req.perm(ticket).require('TICKET_VIEW')
+        req.perm(ticket.resource).require('TICKET_VIEW')
         new_version = int(req.args.get('version', 1))
         old_version = int(req.args.get('old_version', new_version))
         if old_version > new_version:
@@ -1201,15 +1204,17 @@ class TicketModule(Component):
                                "%(message)s", message=to_unicode(e)))
 
         # Redirect the user to the newly created ticket or add attachment
+        ticketref=tag.a('#', ticket.id, href=req.href.ticket(ticket.id))
         if 'attachment' in req.args:
+            add_notice(req, tag_("The ticket %(ticketref)s has been created. "
+                                 "You can now attach the desired files.",
+                                 ticketref=ticketref))
             req.redirect(req.href.attachment('ticket', ticket.id,
                                              action='new'))
         if 'TICKET_VIEW' not in req.perm('ticket', ticket.id):
-            ticket_href = req.href.ticket(ticket.id)
-            add_notice(req, tag_("Your ticket %(ticketref)s has been created, "
+            add_notice(req, tag_("The ticket %(ticketref)s has been created, "
                                  "but you don't have permission to view it.",
-                                 ticketref=tag.a('#', ticket.id,
-                                                 href=ticket_href)))
+                                 ticketref=ticketref))
             req.redirect(req.href.newticket())
         req.redirect(req.href.ticket(ticket.id))
 
@@ -1269,7 +1274,7 @@ class TicketModule(Component):
                                     'label': field_labels.get(field, field)}
         # Start with user changes
         for field, value in ticket._old.iteritems():
-            store_change(field, value, ticket[field], 'user')
+            store_change(field, value or '', ticket[field], 'user')
             
         # Apply controller changes corresponding to the selected action
         problems = []
@@ -1311,7 +1316,7 @@ class TicketModule(Component):
         args[name] = value
         return tag.a(text or value, href=req.href.query(args))
 
-    def _query_link_words(self, req, name, value):
+    def _query_link_words(self, context, name, value):
         """Splits a list of words and makes a query link to each separately"""
         if not isinstance(value, basestring): # None or other non-splitable
             return value
@@ -1319,13 +1324,19 @@ class TicketModule(Component):
                         self.ticketlink_query[1:] or self.ticketlink_query
         args = arg_list_to_args(parse_arg_list(default_query))
         items = []
-        for (i, word) in enumerate(re.split(r'(\s*(?:\s|[,;])\s*)', value)):
+        for i, word in enumerate(re.split(r'([;,\s]+)', value)):
             if i % 2:
                 items.append(word)
             elif word:
-                word_args = args.copy()
-                word_args[name] = '~' + word
-                items.append(tag.a(word, href=req.href.query(word_args)))
+                rendered = name != 'cc' and word \
+                           or Chrome(self.env).format_emails(context, word)
+                if rendered == word:
+                    word_args = args.copy()
+                    word_args[name] = '~' + word
+                    items.append(tag.a(word,
+                                       href=context.href.query(word_args)))
+                else:
+                    items.append(rendered)
         return tag(items)
 
     def _prepare_fields(self, req, ticket):
@@ -1368,12 +1379,11 @@ class TicketModule(Component):
                 field['rendered'] = render_resource_link(self.env, context,
                                                          milestone, 'compact')
             elif name == 'keywords':
-                field['rendered'] = self._query_link_words(
-                                                req, name, ticket[name])
+                field['rendered'] = self._query_link_words(context, name,
+                                                           ticket[name])
             elif name == 'cc':
-                emails = Chrome(self.env).format_emails(context, ticket[name])
-                field['rendered'] = emails == ticket[name] and \
-                        self._query_link_words(req, name, emails) or emails
+                field['rendered'] = self._query_link_words(context, name,
+                                                           ticket[name])
                 if ticket.exists and \
                         'TICKET_EDIT_CC' not in req.perm(ticket.resource):
                     cc = ticket._old.get('cc', ticket['cc'])
