@@ -341,6 +341,15 @@ def copytree(src, dst, symlinks=False, skip=[], overwrite=False):
     copytree_rec(str_path(src), str_path(dst))
 
 
+def is_path_below(path, parent):
+    """Return True iff `path` is equal to parent or is located below `parent`
+    at any level.
+    """
+    path = os.path.abspath(path)
+    parent = os.path.abspath(parent)
+    return path == parent or path.startswith(parent + os.sep)
+
+
 # -- sys utils
 
 def arity(f):
@@ -546,8 +555,10 @@ def get_pkginfo(dist):
 
 # -- crypto utils
 
+_entropy = random.Random()
+
 def hex_entropy(bytes=32):
-    return sha1(str(random.random())).hexdigest()[:bytes]
+    return sha1(str(_entropy.random())).hexdigest()[:bytes]
 
 
 # Original license for md5crypt:
@@ -698,11 +709,18 @@ class Ranges(object):
         self.appendrange(r)
 
     def appendrange(self, r):
-        """Add a range (from a string or None) to the current one"""
+        """Add ranges to the current one. 
+
+        A range is specified as a string of the form "low-high", and 
+        `r` can be a list of such strings, a string containing comma-separated
+        ranges, or `None`.
+        """
         if not r:
             return
         p = self.pairs
-        for x in r.split(","):
+        if isinstance(r, basestring):
+            r = r.split(',')
+        for x in r:
             try:
                 a, b = map(int, x.split('-', 1))
             except ValueError:
@@ -899,7 +917,7 @@ def as_bool(s):
     try:
         return bool(int(s))
     except (TypeError, ValueError):
-        return s.lower() in ('true', 'yes', 'on')
+        return bool(s and s.lower() in ('true', 'yes', 'on'))
 
 def pathjoin(*args):
     """Strip `/` from the arguments and join them with a single `/`."""
