@@ -18,52 +18,57 @@
 #         Matthew Good <trac@matt-good.net>
 #         Christian Boos <cboos@neuf.fr>
 
+import __builtin__
 import locale
 import os
+import re
 import sys
 from urllib import quote, quote_plus, unquote, urlencode
 
+from trac.util.translation import _
+
 
 CRLF = '\r\n'
 
+class Empty(unicode):
+    """A special tag object evaluating to the empty string"""
+    __slots__ = []
+
+empty = Empty()
+
+del Empty # shouldn't be used outside of Trac core
+
+
 # -- Unicode
 
 def to_unicode(text, charset=None):
-    """Convert a `str` object to an `unicode` object.
+    """Convert input to an `unicode` object.
 
-    If `charset` is given, we simply assume that encoding for the text,
-    but we'll use the "replace" mode so that the decoding will always
-    succeed.
-    If `charset` is ''not'' specified, we'll make some guesses, first
-    trying the UTF-8 encoding, then trying the locale preferred encoding,
-    in "replace" mode. This differs from the `unicode` builtin, which
-    by default uses the locale preferred encoding, in 'strict' mode,
-    and is therefore prompt to raise `UnicodeDecodeError`s.
+    For a `str` object, we'll first try to decode the bytes using the given
+    `charset` encoding (or UTF-8 if none is specified), then we fall back to
+    the latin1 encoding which might be correct or not, but at least preserves
+    the original byte sequence by mapping each byte to the corresponding
+    unicode code point in the range U+0000 to U+00FF.
 
-    Because of the "replace" mode, the original content might be altered.
-    If this is not what is wanted, one could map the original byte content
-    by using an encoding which maps each byte of the input to an unicode
-    character, e.g. by doing `unicode(text, 'iso-8859-1')`.
+    Otherwise, a simple `unicode()` conversion is attempted, with some special
+    care taken for `Exception` objects.
     """
-    if not isinstance(text, str):
-        if isinstance(text, Exception):
-            # two possibilities for storing unicode strings in exception data:
-            try:
-                # custom __str__ method on the exception (e.g. PermissionError)
-                return unicode(text)
-            except UnicodeError:
-                # unicode arguments given to the exception (e.g. parse_date)
-                return ' '.join([to_unicode(arg) for arg in text.args])
-        return unicode(text)
-    if charset:
-        return unicode(text, charset, 'replace')
-    else:
+    if isinstance(text, str):
         try:
-            return unicode(text, 'utf-8')
+            return unicode(text, charset or 'utf-8')
+        except UnicodeDecodeError:
+            return unicode(text, 'latin1')
+    elif isinstance(text, Exception):
+        # two possibilities for storing unicode strings in exception data:
+        try:
+            # custom __str__ method on the exception (e.g. PermissionError)
+            return unicode(text)
         except UnicodeError:
-            return unicode(text, locale.getpreferredencoding(), 'replace')
+            # unicode arguments given to the exception (e.g. parse_date)
+            return ' '.join([to_unicode(arg) for arg in text.args])
+    return unicode(text)
 
-def exception_to_unicode(e, traceback=""):
+def exception_to_unicode(e, traceback=False):
     message = '%s: %s' % (e.__class__.__name__, to_unicode(e))
     if traceback:
         from trac.util import get_last_traceback
@@ -71,13 +76,19 @@ def exception_to_unicode(e, traceback=""):
         message = '\n%s\n%s' % (to_unicode('\n'.join(traceback_only)), message)
     return message
 
+_js_quote = {'\\': '\\\\', '"': '\\"', '\b': '\\b', '\f': '\\f',
+             '\n': '\\n', '\r': '\\r', '\t': '\\t', "'": "\\'"}
+for i in range(0x20) + [ord(c) for c in '&<>']:
+    _js_quote.setdefault(chr(i), '\\u%04x' % i)
+_js_quote_re = re.compile(r'[\x00-\x1f\\"\b\f\n\r\t\'&<>]')
+
 def javascript_quote(text):
     """Quote strings for inclusion in javascript"""
     if not text:
         return ''
-    return text.replace('\\', '\\\\').replace('\r', '\\r') \
-               .replace('\n', '\\n').replace('"', '\\"') \
-               .replace("'", "\\'")
+    def replace(match):
+        return _js_quote[match.group(0)]
+    return _js_quote_re.sub(replace, text)
 
 def unicode_quote(value, safe='/'):
     """A unicode aware version of urllib.quote"""
@@ -95,11 +106,23 @@ def unicode_unquote(value):
     return unquote(value).decode('utf-8')
 
 def unicode_urlencode(params):
-    """A unicode aware version of urllib.urlencode"""
+    """A unicode aware version of urllib.urlencode.
+    
+    Values set to `empty` are converted to the key alone, without the
+    equal sign.
+    """
     if isinstance(params, dict):
-        params = params.items()
-    return urlencode([(k, isinstance(v, unicode) and v.encode('utf-8') or v)
-                      for k, v in params])
+        params = params.iteritems()
+    l = []
+    for k, v in params:
+        k = quote_plus(str(k))
+        if v is empty:
+            l.append(k)
+        elif isinstance(v, unicode):
+            l.append(k + '=' + quote_plus(v.encode('utf-8')))
+        else:
+            l.append(k + '=' + quote_plus(str(v)))
+    return '&'.join(l)
 
 def to_utf8(text, charset='iso-8859-15'):
     """Convert a string to UTF-8, assuming the encoding is either UTF-8, ISO
@@ -126,13 +149,25 @@ class unicode_passwd(unicode):
     def __repr__(self):
         return '*******'
 
-def console_print(out, *args):
+def console_print(out, *args, **kwargs):
     cons_charset = getattr(out, 'encoding', None)
     # Windows returns 'cp0' to indicate no encoding
     if cons_charset in (None, 'cp0'):
         cons_charset = 'utf-8'
     out.write(' '.join([to_unicode(a).encode(cons_charset, 'replace') 
-                        for a in args])+ '\n')
+                        for a in args]))
+    if kwargs.get('newline', True):
+        out.write('\n')
+
+def printout(*args, **kwargs):
+    console_print(sys.stdout, *args, **kwargs)
+
+def printerr(*args, **kwargs):
+    console_print(sys.stderr, *args, **kwargs)
+
+def raw_input(prompt):
+    printout(prompt, newline=False)
+    return to_unicode(__builtin__.raw_input(), sys.stdin.encoding)
 
 # -- Plain text formatting
 
@@ -207,15 +242,39 @@ def obfuscate_email_address(address):
                    (address[-1] == '>' and '>' or '')
     return address
 
+def breakable_path(path):
+    """Make a path breakable after path separators, and conversely, avoid
+    breaking at spaces.
+    """
+    if not path:
+        return path
+    prefix = ''
+    if path.startswith('/'):    # Avoid breaking after a leading /
+        prefix = '/'
+        path = path[1:]
+    return prefix + path.replace('/', u'/\u200b').replace('\\', u'\\\u200b') \
+                        .replace(' ', u'\u00a0')
+
+def normalize_whitespace(text, to_space=u'\u00a0', remove=u'\u200b'):
+    """Normalize whitespace in a string, by replacing special spaces by normal
+    spaces and removing zero-width spaces."""
+    if not text:
+        return text
+    for each in to_space:
+        text = text.replace(each, ' ')
+    for each in remove:
+        text = text.replace(each, '')
+    return text
+
 # -- Conversion
 
 def pretty_size(size, format='%.1f'):
     if size is None:
         return ''
 
-    jump = 512
+    jump = 1024
     if size < jump:
-        return '%d bytes' % size
+        return _('%(size)s bytes', size=size)
 
     units = ['KB', 'MB', 'GB', 'TB']
     i = 0
@@ -226,8 +285,10 @@ def pretty_size(size, format='%.1f'):
     return (format + ' %s') % (size, units[i - 1])
 
 def expandtabs(s, tabstop=8, ignoring=None):
-    if '\t' not in s: return s
-    if ignoring is None: return s.expandtabs(tabstop)
+    if '\t' not in s:
+        return s
+    if ignoring is None:
+        return s.expandtabs(tabstop)
 
     outlines = []
     for line in s.split('\n'):
@@ -238,9 +299,9 @@ def expandtabs(s, tabstop=8, ignoring=None):
         s = []
         for c in line:
             if c == '\t':
-                n = tabstop-p%tabstop
-                s.append(' '*n)
-                p+=n
+                n = tabstop - p % tabstop
+                s.append(' ' * n)
+                p += n
             elif not ignoring or c not in ignoring:
                 p += 1
                 s.append(c)
