@@ -101,8 +101,10 @@ class NullTranslationsBabel(NullTranslations):
     def dungettext(self, domain, singular, plural, num):
         return self.ungettext(singular, plural, num)
 
+has_babel = False
 
 try:
+    from babel import Locale
     from babel.support import LazyProxy, Translations
 
     class TranslationsProxy(object):
@@ -139,8 +141,8 @@ try:
         def activate(self, locale, env_path=None):
             try:
                 locale_dir = pkg_resources.resource_filename('trac', 'locale')
-            except pkg_resources.ExtractionError:
-                return # delay extraction
+            except Exception:
+                return
             t = Translations.load(locale_dir, locale or 'en_US')
             if not t or t.__class__ is NullTranslations:
                 t = self._null_translations
@@ -325,9 +327,20 @@ try:
         """Return a list of locale identifiers of the locales for which
         translations are available.
         """
-        return [dirname for dirname
-                in pkg_resources.resource_listdir('trac', 'locale')
-                if '.' not in dirname]
+        try:
+            return [dirname for dirname
+                    in pkg_resources.resource_listdir('trac', 'locale')
+                    if '.' not in dirname]
+        except Exception:
+            return []
+
+    def get_negotiated_locale(preferred_locales):
+        def normalize(locale_ids):
+            return [id.replace('_', '-') for id in locale_ids if id]
+        return Locale.negotiate(normalize(preferred_locales),
+                                normalize(get_available_locales()), sep='-')
+        
+    has_babel = True
 
 except ImportError: # fall back on 0.11 behavior, i18n functions are no-ops
     gettext = _ = gettext_noop
@@ -358,3 +371,6 @@ except ImportError: # fall back on 0.11 behavior, i18n functions are no-ops
 
     def get_available_locales():
         return []
+
+    def get_negotiated_locale(preferred=None, default=None):
+        return None
