@@ -30,12 +30,6 @@ from trac.versioncontrol.api import RepositoryManager
 
 
 def parent_iter(path):
-    path = path.strip('/')
-    if path:
-        path = '/' + path + '/'
-    else:
-        path = '/'
-
     while 1:
         yield path
         if path == '/':
@@ -46,15 +40,21 @@ def parent_iter(path):
         path = path[:idx + 1]
 
 
+def join(*args):
+    args = (arg.strip('/') for arg in args)
+    return '/'.join(arg for arg in args if arg)
+
+
 class ParseError(Exception):
     """Exception thrown for parse errors in authz files"""
 
 
-def parse(authz):
+def parse(authz, modules):
     """Parse a Subversion authorization file.
     
     Return a dict of modules, each containing a dict of paths, each containing
-    a dict mapping users to permissions.
+    a dict mapping users to permissions. Only modules contained in `modules`
+    are retained.
     """
     groups = {}
     aliases = {}
@@ -84,7 +84,10 @@ def parse(authz):
         elif section == 'aliases':
             aliases[name] = value.strip()
         else:
-            sections.setdefault(section, []).append((name.strip(), value))
+            parts = section.split(':', 1)
+            module, path = len(parts) > 1 and parts[0] or '', parts[-1]
+            if module in modules:
+                sections.setdefault((module, path), []).append((name, value))
 
     def resolve(subject, done):
         if subject.startswith('@'):
@@ -98,10 +101,8 @@ def parse(authz):
             yield subject
     
     authz = {}
-    for name, items in sections.iteritems():
-        parts = name.split(':', 1)
-        module = authz.setdefault(len(parts) > 1 and parts[0] or '', {})
-        section = module.setdefault(parts[-1], {})
+    for (module, path), items in sections.iteritems():
+        section = authz.setdefault(module, {}).setdefault(path, {})
         for subject, perms in items:
             for user in resolve(subject, set()):
                 section.setdefault(user, 'r' in perms)  # The first match wins
@@ -131,52 +132,78 @@ class AuthzSourcePolicy(Component):
 
     authz_module_name = Option('trac', 'authz_module_name', '',
         """The module prefix used in the `authz_file` for the default
-        repository. If left empty, the global sections will be used.
+        repository. If left empty, the global section is used.
         """)
 
     _mtime = 0
     _authz = {}
     _users = set()
     
+    _handled_perms = frozenset([(None, 'BROWSER_VIEW'),
+                                (None, 'CHANGESET_VIEW'),
+                                (None, 'FILE_VIEW'),
+                                (None, 'LOG_VIEW'),
+                                ('source', 'BROWSER_VIEW'),
+                                ('source', 'FILE_VIEW'),
+                                ('source', 'LOG_VIEW'),
+                                ('changeset', 'CHANGESET_VIEW')])
+
     # IPermissionPolicy methods
 
     def check_permission(self, action, username, resource, perm):
-        if action == 'FILE_VIEW' or action == 'BROWSER_VIEW':
+        realm = resource and resource.realm or None
+        if (realm, action) in self._handled_perms:
             authz, users = self._get_authz_info()
             if authz is None:
                 return False
+            
+            if username == 'anonymous':
+                usernames = ('$anonymous', '*')
+            else:
+                usernames = (username, '$authenticated', '*')
             if resource is None:
-                return users is True or username in users
-            if resource.realm == 'source':
-                modules = [resource.parent.id or self.authz_module_name]
-                if modules[0]:
-                    modules.append('')
-                for p in parent_iter(resource.id):
-                    for module in modules:
-                        section = authz.get(module, {}).get(p, {})
-                        result = section.get(username)
-                        if result is not None:
-                            return result
-                        result = section.get('*')
-                        if result is not None:
-                            return result
-                return False
-        elif action == 'CHANGESET_VIEW':
-            authz, users = self._get_authz_info()
-            if authz is None:
-                return False
-            if resource is None:
-                return users is True or username in users
-            if resource.realm == 'changeset':
-                rm = RepositoryManager(self.env)
+                return users & set(usernames) and True or None
+
+            rm = RepositoryManager(self.env)
+            try:
                 repos = rm.get_repository(resource.parent.id)
+            except TracError:
+                return True # Allow error to be displayed in the repo index
+            modules = [resource.parent.id or self.authz_module_name]
+            if modules[0]:
+                modules.append('')
+
+            def check_path(path):
+                path = '/' + join(repos.scope, path)
+                if path != '/':
+                    path += '/'
+                
+                # Walk from resource up parent directories
+                for spath in parent_iter(path):
+                    for module in modules:
+                        section = authz.get(module, {}).get(spath)
+                        if section:
+                            for user in usernames:
+                                result = section.get(user)
+                                if result is not None:
+                                    return result
+                
+                # Allow access to parent directories of allowed resources
+                if any(section.get(user) is True
+                       for module in modules
+                       for spath, section in authz.get(module, {}).iteritems()
+                       if spath.startswith(path)
+                       for user in usernames):
+                    return True
+            
+            if realm == 'source':
+                return check_path(resource.id)
+
+            elif realm == 'changeset':
                 changes = list(repos.get_changeset(resource.id).get_changes())
-                if not changes:
+                if not changes or any(check_path(change[0])
+                                      for change in changes):
                     return True
-                source = Resource('source', version=resource.id,
-                                  parent=resource.parent)
-                return any('FILE_VIEW' in perm(source(id=change[0]))
-                           for change in changes)
 
     def _get_authz_info(self):
         try:
@@ -190,13 +217,19 @@ class AuthzSourcePolicy(Component):
             self._users = set()
         if mtime > self._mtime:
             self._mtime = mtime
+            rm = RepositoryManager(self.env)
+            modules = set(repos.reponame
+                          for repos in rm.get_real_repositories())
+            if '' in modules and self.authz_module_name:
+                modules.add(self.authz_module_name)
+            modules.add('')
             self.log.info('Parsing authz file: %s' % self.authz_file)
             try:
-                self._authz = parse(read_file(self.authz_file))
-                users = set(user for module in self._authz.itervalues()
-                            for path in module.itervalues()
-                            for user, result in path.iteritems() if result)
-                self._users = '*' in users or users
+                self._authz = parse(read_file(self.authz_file), modules)
+                self._users = set(user for paths in self._authz.itervalues()
+                                  for path in paths.itervalues()
+                                  for user, result in path.iteritems()
+                                  if result)
             except Exception, e:
                 self._authz = None
                 self._users = set()
