@@ -17,6 +17,7 @@
 
 from datetime import datetime, timedelta
 from fnmatch import fnmatchcase
+import inspect
 import re
 
 from genshi.builder import tag
@@ -26,10 +27,10 @@ from trac.core import *
 from trac.mimeview.api import Mimeview, is_binary, \
                               IHTMLPreviewAnnotator, Context
 from trac.perm import IPermissionRequestor
-from trac.resource import ResourceNotFound
-from trac.util import embedded_numbers
+from trac.resource import Resource, ResourceNotFound
+from trac.util import as_bool, embedded_numbers
 from trac.util.compat import any
-from trac.util.datefmt import http_date, utc
+from trac.util.datefmt import http_date, to_datetime, utc
 from trac.util.html import escape, Markup
 from trac.util.text import exception_to_unicode, shorten_line
 from trac.util.translation import _
@@ -291,9 +292,7 @@ class BrowserModule(Component):
 
     def get_navigation_items(self, req):
         rm = RepositoryManager(self.env)
-        if any(repos and repos.params.get('hidden') not in _TRUE_VALUES
-               and repos.can_view(req.perm)
-               for repos in rm.get_real_repositories()):
+        if 'BROWSER_VIEW' in req.perm and rm.get_real_repositories():
             yield ('mainnav', 'browser',
                    tag.a(_('Browse Source'), href=req.href.browser()))
 
@@ -324,9 +323,11 @@ class BrowserModule(Component):
             return True
 
     def process_request(self, req):
-        go_to_preselected = req.args.get('preselected')
-        if go_to_preselected:
-            req.redirect(go_to_preselected)
+        req.perm.require('BROWSER_VIEW')
+
+        presel = req.args.get('preselected')
+        if presel and (presel + '/').startswith(req.href.browser() + '/'):
+            req.redirect(presel)
 
         path = req.args.get('path', '/')
         rev = req.args.get('rev', '')
@@ -337,13 +338,11 @@ class BrowserModule(Component):
         xhr = req.get_header('X-Requested-With') == 'XMLHttpRequest'
         
         rm = RepositoryManager(self.env)
+        all_repositories = rm.get_all_repositories()
         reponame, repos, path = rm.get_repository_by_path(path)
 
         # Repository index
-        all_repositories, repoinfo = None, None
         if not reponame and path == '/':
-            all_repositories = rm.get_all_repositories()
-            repoinfo = all_repositories.get(reponame)
             if repos and (all_repositories[''].get('hidden') in _TRUE_VALUES
                           or not repos.can_view(req.perm)):
                 repos = None
@@ -383,15 +382,18 @@ class BrowserModule(Component):
                                     order, desc)
 
         repo_data = dir_data = file_data = None
-        if all_repositories:
+        if not reponame and path == '/':
             repo_data = self._render_repository_index(
-                    context, all_repositories, order, desc)
+                                        context, all_repositories, order, desc)
         if node:
             if node.isdir:
                 dir_data = self._render_dir(req, repos, node, rev, order, desc)
             elif node.isfile:
                 file_data = self._render_file(req, context, repos, node, rev)
 
+        if not repos and not (repo_data and repo_data['repositories']):
+            raise ResourceNotFound(_("No node %(path)s", path=path))
+
         quickjump_data = properties_data = None
         if node and not xhr:
             properties_data = self.render_properties(
@@ -400,7 +402,7 @@ class BrowserModule(Component):
 
         data = {
             'context': context, 'reponame': reponame, 'repos': repos,
-            'repoinfo': repoinfo,
+            'repoinfo': all_repositories.get(reponame or ''),
             'path': path, 'rev': node and node.rev, 'stickyrev': rev,
             'display_rev': display_rev,
             'created_path': node and node.created_path,
@@ -487,7 +489,7 @@ class BrowserModule(Component):
 
         rm = RepositoryManager(self.env)
         repositories = []
-        for reponame, repoinfo in all_repositories.items():
+        for reponame, repoinfo in all_repositories.iteritems():
             if not reponame or repoinfo.get('hidden') in _TRUE_VALUES:
                 continue
             try:
@@ -495,7 +497,10 @@ class BrowserModule(Component):
                 if repos:
                     if not repos.can_view(context.perm):
                         continue
-                    youngest = repos.get_changeset(repos.youngest_rev)
+                    try:
+                        youngest = repos.get_changeset(repos.youngest_rev)
+                    except NoSuchChangeset:
+                        youngest = None
                     if self.color_scale and youngest:
                         if not timerange:
                             timerange = TimeRange(youngest.date)
@@ -503,16 +508,25 @@ class BrowserModule(Component):
                             timerange.insert(youngest.date)
                     entry = (reponame, repoinfo, repos, youngest, None)
                 else:
-                    entry = (reponame, repoinfo, None, None, "XXX")
+                    entry = (reponame, repoinfo, None, None, u"\u2013")
             except TracError, err:
                 entry = (reponame, repoinfo, None, None,
                          exception_to_unicode(err))
+            if entry[-1] is not None:   # Check permission in case of error
+                root = Resource('repository', reponame).child('source', '/')
+                if 'BROWSER_VIEW' not in context.perm(root):
+                    continue
             repositories.append(entry)
 
         # Ordering of repositories
         if order == 'date':
             def repo_order((reponame, repoinfo, repos, youngest, err)):
-                return youngest and youngest.date
+                return (youngest and youngest.date or to_datetime(0),
+                        embedded_numbers(reponame.lower()))
+        elif order == 'author':
+            def repo_order((reponame, repoinfo, repos, youngest, err)):
+                return (youngest and youngest.author.lower() or '',
+                        embedded_numbers(reponame.lower()))
         else:
             def repo_order((reponame, repoinfo, repos, youngest, err)):
                 return embedded_numbers(reponame.lower())
@@ -534,7 +548,7 @@ class BrowserModule(Component):
                 
         entries = [entry(n) for n in node.get_entries()
                    if n.can_view(req.perm)]
-        changes = get_changes(repos, [i.rev for i in entries])
+        changes = get_changes(repos, [i.rev for i in entries], self.log)
 
         if rev:
             newest = repos.get_changeset(rev).date
@@ -557,14 +571,16 @@ class BrowserModule(Component):
         # Ordering of entries
         if order == 'date':
             def file_order(a):
-                return changes[a.rev].date
+                return (changes[a.rev].date,
+                        embedded_numbers(a.name.lower()))
         elif order == 'size':
             def file_order(a):
                 return (a.content_length,
                         embedded_numbers(a.name.lower()))
         elif order == 'author':
             def file_order(a):
-                return changes[a.rev].author.lower()
+                return (changes[a.rev].author.lower(),
+                        embedded_numbers(a.name.lower()))
         else:
             def file_order(a):
                 return embedded_numbers(a.name.lower())
@@ -778,29 +794,35 @@ class BrowserModule(Component):
         yield "RepositoryIndex"
 
     def get_macro_description(self, name):
-        return """
+        return inspect.cleandoc("""
         Display the list of available repositories.
 
-        Can be given a ''format'' argument (defaults to ''compact'')
-         - ''compact'' will produce a comma-separated list of
-           repository prefix names 
-         - ''list'' will produce a description list of
-           repository prefix names 
-         - ''table'' will produce a table view, similar to the 
-           one visible in the ''Browse View'' page
+        Can be given the following named arguments:
         
-        Can be given a ''glob'' argument, which will do a glob-style
-        filtering on the repository names (defaults to '*')
+          ''format''::
+            Select the rendering format:
+            - ''compact'' produces a comma-separated list of repository prefix
+              names (default)
+            - ''list'' produces a description list of repository prefix names 
+            - ''table'' produces a table view, similar to the one visible in
+              the ''Browse View'' page
+          ''glob''::
+            Do a glob-style filtering on the repository names (defaults to '*')
+          ''order''::
+            Order repositories by the given column (one of "name", "date" or
+            "author")
+          ''desc''::
+            When set to 1, order by descending order
 
-        (since 0.12)
-        """
+        (''since 0.12'')
+        """)
 
     def expand_macro(self, formatter, name, content):
         args, kwargs = parse_args(content)
         format = kwargs.get('format', 'compact')
         glob = kwargs.get('glob', '*')
         order = kwargs.get('order')
-        desc = kwargs.get('desc', 0)
+        desc = as_bool(kwargs.get('desc', 0))
 
         rm = RepositoryManager(self.env)
         all_repos = dict(rdata for rdata in rm.get_all_repositories().items()
@@ -811,26 +833,36 @@ class BrowserModule(Component):
                                                  order, desc)
 
             add_stylesheet(formatter.req, 'common/css/browser.css')
-            data = {'repo': repo, 'desc': desc and 1 or None,
-                    'reponame': None, 'path': '/', 'stickyrev': None}
+            wiki_format_messages = self.config['changeset'] \
+                                       .getbool('wiki_format_messages')
+            data = {'repo': repo, 'order': order, 'desc': desc and 1 or None,
+                    'reponame': None, 'path': '/', 'stickyrev': None,
+                    'wiki_format_messages': wiki_format_messages}
             from trac.web.chrome import Chrome
             return Chrome(self.env).render_template(
                     formatter.req, 'repository_index.html', data, None,
                     fragment=True)
 
+        def get_repository(reponame):
+            try:
+                return rm.get_repository(reponame)
+            except TracError:
+                return
+
+        all_repos = [(reponame, get_repository(reponame))
+                     for reponame in all_repos]
+        all_repos = sorted(((reponame, repos) for reponame, repos in all_repos
+                            if repos
+                            and repos.params.get('hidden') not in _TRUE_VALUES
+                            and repos.can_view(formatter.perm)),
+                           reverse=desc)
+
         def repolink(reponame, repos):
             label = reponame or _('(default)')
             return Markup(tag.a(label, 
                           title=_('View repository %(repo)s', repo=label),
                           href=formatter.href.browser(repos.reponame or None)))
 
-        all_repos = dict((reponame, rm.get_repository(reponame))
-                         for reponame in all_repos)
-        all_repos = sorted((reponame, repos) for reponame, repos in all_repos
-                           if repos
-                           and repos.params.get('hidden') not in _TRUE_VALUE
-                           and repos.can_view(formatter.perm))
-
         if format == 'list':
             return tag.dl([
                 tag(tag.dt(repolink(reponame, repos)),
