@@ -1037,7 +1037,7 @@ class ChangesetModule(Component):
             # + optional query and fragment
             r"%s(?:/[^\]]*)?(?:\?[^\]]*)?(?:#[^\]]*)?\]|" % self.CHANGESET_ID +
             # r... form: allow r1 but not r1:2 (handled by the log syntax)
-            r"(?:\b|!)r\d+\b(?!:\d)",
+            r"(?:\b|!)r\d+\b(?!:\d)(?:/[a-zA-Z0-9_/+-]+)?",
             lambda x, y, z:
             self._format_changeset_link(x, 'changeset',
                                         y[0] == 'r' and y[1:] or y[1:-1],
@@ -1062,11 +1062,11 @@ class ChangesetModule(Component):
             rev, path = chgset[:sep], chgset[sep:]
         else:
             rev, path = chgset, '/'
-        reponame = rm.get_default_repository(formatter.context)
-        if reponame is not None:
-            repos = rm.get_repository(reponame)
-        else:
-            reponame, repos, path = rm.get_repository_by_path(path)
+        reponame, repos, path = rm.get_repository_by_path(path)
+        if not reponame:
+            reponame = rm.get_default_repository(formatter.context)
+            if reponame is not None:
+                repos = rm.get_repository(reponame)
         if path == '/':
             path = None
 
