@@ -230,17 +230,10 @@ class LogModule(Component):
             info[-1]['change'] = None
         
         revisions = [i['rev'] for i in info]
-        changes = get_changes(repos, revisions)
+        changes = get_changes(repos, revisions, self.log)
         extra_changes = {}
-        email_map = {}
         
-        if format == 'rss':
-            # Get the email addresses of all known users
-            if Chrome(self.env).show_email_addresses:
-                for username, name, email in self.env.get_known_users():
-                    if email:
-                        email_map[username] = email
-        elif format == 'changelog':
+        if format == 'changelog':
             for rev in revisions:
                 changeset = changes[rev]
                 cs = {}
@@ -263,15 +256,15 @@ class LogModule(Component):
             'path': path, 'rev': rev, 'stop_rev': stop_rev,
             'display_rev': display_rev, 'revranges': revranges,
             'mode': mode, 'verbose': verbose, 'limit' : limit,
-            'items': info, 'changes': changes,
-            'email_map': email_map, 'extra_changes': extra_changes,
+            'items': info, 'changes': changes, 'extra_changes': extra_changes,
             'wiki_format_messages':
             self.config['changeset'].getbool('wiki_format_messages')
         }
 
-        if req.args.get('format') == 'changelog':
+        if format == 'changelog':
             return 'revisionlog.txt', data, 'text/plain'
-        elif req.args.get('format') == 'rss':
+        elif format == 'rss':
+            data['email_map'] = Chrome(self.env).get_email_map()
             data['context'] = Context.from_request(req, 'source', 
                                                    path, parent=repos.resource,
                                                    absurls=True)
@@ -331,8 +324,8 @@ class LogModule(Component):
             r"(?P<log_revs>%s)(?P<log_path>[/?][^\]]*)?\]" % self.REV_RANGE,
             lambda x, y, z: self._format_link(x, 'log1', y[1:-1], y, z))
         yield (
-            # r<from>:<to> form (no intertrac and no path restriction)
-            r"(?:\b|!)r%s\b" % Ranges.RE_STR,
+            # r<from>:<to> form + optional path restriction (no intertrac)
+            r"(?:\b|!)r%s\b(?:/[a-zA-Z0-9_/+-]+)?" % Ranges.RE_STR,
             lambda x, y, z: self._format_link(x, 'log2', '@' + y[1:], y))
 
     def get_link_resolvers(self):
@@ -357,6 +350,7 @@ class LogModule(Component):
                 match, query, fragment = formatter.split_link(match)
             else:
                 query = fragment = ''
+                match = ''.join(reversed(match.split('/', 1)))
             path = match
             revs = ''
             if self.LOG_LINK_RE.match(match):
@@ -365,11 +359,11 @@ class LogModule(Component):
                 path, revs = match[:idx], match[idx+1:]
         
         rm = RepositoryManager(self.env)
-        reponame = rm.get_default_repository(formatter.context)
-        if reponame is not None:
-            repos = rm.get_repository(reponame)
-        else:
-            reponame, repos, path = rm.get_repository_by_path(path)
+        reponame, repos, path = rm.get_repository_by_path(path)
+        if not reponame:
+            reponame = rm.get_default_repository(formatter.context)
+            if reponame is not None:
+                repos = rm.get_repository(reponame)
 
         if repos:
             revranges = None
