@@ -318,9 +318,9 @@ class PageOutlineMacro(WikiMacroBase):
         # TODO: - integrate the rest of the OutlineFormatter directly here
         #       - use formatter.wikidom instead of formatter.source
         out = StringIO()
-        OutlineFormatter(self.env, formatter.context).format(formatter.source,
-                                                             out, max_depth,
-                                                             min_depth)
+        oformatter = OutlineFormatter(self.env, formatter.context)
+        oformatter.format(formatter.source, out, max_depth, min_depth,
+                          shorten=not inline)
         outline = Markup(out.getvalue())
 
         if title:
